#!/bin/tcsh
# script to process Brodmann atlas prepared by Jurgen Mai and Milan Matajnik
# 
set ba_atlas = Brodmann_Mai_Matajnik.nii.gz
set master_template = ~/abin/MNI152_2009_template.nii.gz
setenv AFNI_SUPP_ATLAS_DIR `pwd`/Brodmann_MM
# goto RESTART

\rm temp* $ba_atlas ?h*ribbon*

3dcopy BA_smooth_fill_v002s7o4.nii.gz tempba.nii.gz
3drefit -cmap INT_CMAP -space MNI tempba.nii.gz

# copy ribbon masks from MNI2009c surface data (Freesurfer, @SUMA_MakeSpecFS)
set surfdir = ~/schaefer_yeo/suma_MNI152_2009/
cp  $surfdir/*ribbon* .

# remove insula from ribbon - largest cluster of voxels not identified in atlas
foreach hemi ( rh lh )
   # resample FreeSurfer cortical ribbons to MNI template
   3dresample -prefix $hemi.ribbon_rs.nii.gz -overwrite -master $master_template \
      -rmode NN -inset $hemi.ribbon.nii
   # find unidentified ribbon regions
   3dcalc -a tempba.nii.gz -b $hemi.ribbon_rs.nii.gz -expr 'not(a)*step(b)' \
      -prefix $hemi.ufo_ribbon.nii.gz -overwrite
   # any cluster of unidenitified regions greater than 50 voxels is intentional,
   # so keep it. mostly it seems to be insula
   3dClusterize                          \
      -inset   $hemi.ufo_ribbon.nii.gz \
      -ithr 0                          \
      -idat 0                          \
      -mask $hemi.ufo_ribbon.nii.gz    \
      -NN 3                            \
      -1sided RIGHT_TAIL 1             \
      -clust_nvox 50                   \
      -overwrite                       \
      -pref_map $hemi.insula.ribbon.nii.gz
   # make non-insula ribbon
   3dcalc -a $hemi.insula.ribbon.nii.gz -b $hemi.ribbon_rs.nii.gz -expr 'not(a)*step(b)' \
      -prefix $hemi.limited_ribbon.nii.gz -overwrite
   # dilate the ribbons by 1 voxel
   3dmask_tool -overwrite -dilate_result 1 -prefix $hemi.d1.ribbon.nii.gz \
      -overwrite -input $hemi.limited_ribbon.nii.gz
end

# rm -rf $AFNI_SUPP_ATLAS_DIR

# extract labels
# 43 Brodmann regions with indices 1-43
# extract from ITK labels
# format for each line is
# index r g b 1 1 1 label
\rm temp_balabels.txt
set nba = 43
tail -$nba mrx_cT_BrodmanAreas_ver002.label > balabels.txt
foreach li (`count 1 43` )
   set ll = `column_cat -line $li balabels.txt`
   set lv = `echo $ll[8]|tr -d \" `
   echo "${ll[1]}  right_$lv" >> temp_balabels.txt
   set llv = `ccalc -int "${ll[1]} + 100"`
   echo "$llv  left_$lv" >> temp_balabels.txt
end
sort -n temp_balabels.txt > balabels_lr.txt

# make right side 1-43, left side 101-143
3dcalc -a tempba.nii.gz -b lh.d1.ribbon.nii.gz -c rh.d1.ribbon.nii.gz \
   -expr '(a+100)*step(a)*step(b) + a*not(b)*step(c)' \
   -prefix tempbalr.nii.gz  -overwrite

####################################################################################
# method 0 - keep original in its own mask, but add 100 to left side
3dcalc -a tempba.nii.gz -b lh.limited_ribbon.nii.gz -c rh.limited_ribbon.nii.gz \
   -expr '(a+100)*step(a)*step(b) + a*not(b)*step(c)' \
   -prefix BAlr_orig.nii.gz  -overwrite

@Atlasize -atlas_type G -centers -centertype Icent -dset BAlr_orig.nii.gz \
   -lab_file balabels_lr.txt 1 0 -auto_backup -space MNI \
   -atlas_name Brodmann_MMorig -atlas_description "Brodmann-orig,Mai and Matajnik v1.0, 2020"
# make surfaces
rm -rf surfs_orig
mkdir surfs_orig
cd surfs_orig
IsoSurface -isorois+dsets -o BA.gii -input ../BAlr_orig.nii.gz 
cd ..

####################################################################################
# 4 or 5 methods to dilate regions to dilated mask
####################################################################################
# method 1 - dilate with iterative non-zero modal smoothing and remove small clusters
# then refill again the same way

# iteratively dilate regions with non-zero modal smoothing and remove lost voxels
rm -rf rmgrow0/
@ROI_modal_grow -niters 5 -input tempbalr.nii.gz -mask both_ribbons.d1.nii.gz \
    -prefix BA_rmg0 -outdir rmgrow0
rm -rf roidc/
@ROI_decluster -frac_thresh 0.15 -input rmgrow0/BA_rmg0.nii.gz \
    -prefix BA_rdc -outdir roidc
rm -rf rmgrow/
@ROI_modal_grow -niters 5 -input roidc/BA_rdc.nii.gz -mask both_ribbons.d1.nii.gz \
    -prefix BA_rmg -outdir rmgrow
3dcopy -overwrite rmgrow/BA_rmg.nii.gz ./
# make atlas dataset - calculate centers for all regions
@Atlasize -atlas_type G -centers -centertype Icent -dset BA_rmg.nii.gz \
   -lab_file balabels_lr.txt 1 0 -auto_backup -space MNI \
   -atlas_name Brodmann_MMrmg -atlas_description "Mai and Matajnik v1.0, 2020"
# make surfaces
rm -rf surfs_rmg
mkdir surfs_rmg
cd surfs_rmg
IsoSurface -isorois+dsets -o BA.gii -input ../BA_rmg.nii.gz 
cd ..

####################################################################################
# method 2 - dilate with non-zero modal smooth non-iteratively, just a larger neighborhood
# dilate the regions with non-zero modal smoothing
3dLocalstat -overwrite -prefix tempbam3.nii.gz -stat nzmode -nbhd 'SPHERE(-3)' \
   tempbalr.nii.gz
# put original atlas voxels back, but fill with nonzero mode to dilated ribbon mask
3dcalc -a tempbalr.nii.gz -b tempbam3.nii.gz -c rh.d1.ribbon.nii.gz -d lh.d1.ribbon.nii.gz \
   -prefix $ba_atlas -expr '(a+not(a)*b)*step(c+d)' -datum byte -nscale -overwrite
# make atlas dataset - calculate centers for all regions
@Atlasize -atlas_type G -centers -centertype Icent -dset $ba_atlas \
   -lab_file balabels_lr.txt 1 0 -auto_backup -space MNI \
   -atlas_name Brodmann_MMnzm -atlas_description "Mai and Matajnik v1.0, 2020"
# make surfaces 
rm -rf surfs_nzm
mkdir surfs_nzm
cd surfs_nzm
IsoSurface -isorois+dsets -o BA.gii -input ../$ba_atlas
cd ..

####################################################################################
# method 3 - project to surface, then back to dilated mask volume, spackling
# with iterative non-zero modal smooth
# variant 1 - use reprojection values only
# variant 2 - restore original voxel values
foreach hemi ( lh rh )
  3dVol2Surf -spec ${surfdir}/std.141.MNI152_2009_${hemi}.spec \
      -surf_A pial -surf_B smoothwm -map_func nzmode -oob_value 0 \
      -out_niml project.${hemi}.niml.dset -f_steps 5 -f_index nodes -overwrite \
      -grid_parent BAlr_orig.nii.gz -sv tempbalr.nii.gz 
  @surf_to_vol_spackle -maskset $hemi.d1.ribbon.nii.gz \
      -spec ${surfdir}/std.141.MNI152_2009_${hemi}.spec \
      -surfA pial -surfB smoothwm -mode -meanrad 1 -maxiters 8 \
      -surfset project.${hemi}.niml.dset -prefix reprojectvol.$hemi 
end

3dMean -max -prefix tempbalr_reproj.nii.gz -overwrite reprojectvol.?h.nii.gz
# put original atlas voxels back
3dcalc -a tempbalr.nii.gz -b tempbalr_reproj.nii.gz  \
       -c rh.d1.ribbon.nii.gz -d lh.d1.ribbon.nii.gz \
   -prefix ba_reproject.nii.gz -expr '(a+not(a)*b)*step(c+d)' -datum byte -nscale -overwrite

# variant 1 -  original voxels
# make atlas dataset - calculate centers for all regions
@Atlasize -atlas_type G -centers -centertype Icent -dset ba_reproject.nii.gz \
   -lab_file balabels_lr.txt 1 0 -auto_backup -space MNI \
   -atlas_name Brodmann_MMrpo -atlas_description "Mai and Matajnik v1.0, 2020, reproject,original replaced"

# variant 2 - reprojected voxels only
# make atlas dataset - calculate centers for all regions
@Atlasize -atlas_type G -centers -centertype Icent -dset tempbalr_reproj.nii.gz \
   -lab_file balabels_lr.txt 1 0 -auto_backup -space MNI \
   -atlas_name Brodmann_MMrp -atlas_description "Mai and Matajnik v1.0, 2020, reproject"
# make surfaces for reprojected with original voxels
rm -rf surfs_reproj
mkdir surfs_reproj
cd surfs_reproj
IsoSurface -isorois+dsets -o BA.gii -input ../ba_reproject.nii.gz
cd ..

# make surfaces just reprojected
rm -rf surfs_reproj_only
mkdir surfs_reproj_only
cd surfs_reproj_only
IsoSurface -isorois+dsets -o BA.gii -input ../tempbalr_reproj.nii.gz
cd ..

####################################################################################
# method 4 - use 3dROImaker for dilation
# 3dROImaker inflation instead of surface projection or modal smoothing
# dilate more than needed but limit to 1 voxel dilated ribbon (not including insula)
3dMean -max -prefix both_ribbons.d1.nii.gz -overwrite lh.d1.ribbon.nii.gz rh.d1.ribbon.nii.gz
3dROIMaker -refset BAlr_orig.nii.gz -inset BAlr_orig.nii.gz \
  -mask both_ribbons.d1.nii.gz -prefix ROImake_dil -inflate 5 -overwrite
3dcopy -overwrite ROImake_dil_GMI+tlrc Brodmann_ROImake.nii.gz
# make atlas dataset - calculate centers for all regions
@Atlasize -atlas_type G -centers -centertype Icent -dset Brodmann_ROImake.nii.gz \
   -lab_file balabels_lr.txt 1 0 -auto_backup -space MNI \
   -atlas_name Brodmann_MM3dRm -atlas_description "Mai and Matajnik v1.0, 2020, 3dROImaker"
# make surfaces
rm -rf surfs_3dROImaker
mkdir surfs_3dROImaker
cd surfs_3dROImaker
IsoSurface -isorois+dsets -o BA.gii -input ../Brodmann_ROImake.nii.gz
cd ..

####################################################################################
# all done with the different methods

exit 0

# questionable regions
BA26 hole and crosses left-right
BA34 hole
BA37 extra
BA24 crosses left-right?
